@@ -2,6 +2,7 @@ import atexit
 import fakeredis
 import queue
 import redis
+import socket
 import subprocess
 import time
 
@@ -21,7 +22,9 @@ class RedisBridge(Loggable):
         - name: the name of this RedisBridge
 
     Methods:
-        - subscribe(channel)
+        - connect(use_mock_redis_server=False, **redis_kwargs)
+        - subscribe(*channels)
+        - unsubscribe(*channels)
         - register(observer, channel)
         - deregister(observer, channel=None)
         - register_callback(callback, channel, message_type=None)
@@ -33,10 +36,13 @@ class RedisBridge(Loggable):
         - respond(data, channel, request_id)
     """
 
-    def __init__(self, name=None, use_mock_redis_server=False, **redis_kwargs):
+    def __init__(
+        self, name=None, connect_on_creation=True, use_mock_redis_server=False, **redis_kwargs):
         """
         Arguments:
             - name: the name of this RedisBridge
+            - connect_on_creation: boolean indicating whether or not to immediately
+                connect to the Redis server
             - use_mock_redis_server: boolean indicating whether or not to use a mock Redis
                 connection that simulates talking to a Redis server by storing state internally
 
@@ -58,46 +64,13 @@ class RedisBridge(Loggable):
         self._server_process = None
         self._callback_interface = CallbackInterface(self)
 
-        # If indicated, use mock Redis server
-        if use_mock_redis_server:
-            self._connect_mock()
-            atexit.register(self._cleanup)
-            return
-
-        # Try to connect to given host & port
-        self._connect(**redis_kwargs)
-
-        # Fallback to connecting to localhost
-        if not self._connection:
-            # Spin up a local Redis server
-            port = redis_kwargs.get('port', 6379)
-            self.logger.warning(f"{self}:  Attempting to spin up Redis server at localhost:{port}")
-            try:
-                self._server_process = subprocess.Popen(['redis-server', '--port', str(port)])
-
-                # Wait for local Redis server
-                timeout = 1.0
-                start = time.time()
-                while check_server('localhost', port):
-                    if time.time() - start > timeout:
-                        self._server_process.terminate()
-                        break
-            except FileNotFoundError as e:
-                self.logger.error(f"{self}:  Could not find executable 'redis-server'.")
-            except Exception as e:
-                self.logger.error(f"{self}:  Could not spin up Redis server - {e}")
-
-
-            # Connect to local Redis server
-            self._connect(**dict(redis_kwargs, host='localhost'))
-
-        # Fallback to using mock Redis server
-        if not self._connection:
-            self._connect_mock()
-
-        # Stop bridge on program termination
+        # Cleanup on program termination
         atexit.register(self._cleanup)
 
+        # Connect to the Redis server
+        if connect_on_creation:
+            self.connect(use_mock_redis_server=use_mock_redis_server, **redis_kwargs)
+
 
     def __str__(self):
         """
@@ -109,15 +82,68 @@ class RedisBridge(Loggable):
             return f"[{self.__class__.__name__}]"
 
 
-    def subscribe(self, channel):
+    def connect(self, use_mock_redis_server=False, **redis_kwargs):
         """
-        Subscribe to messages from a specific channel through the Redis connection.
+        Connect to the Redis server.
 
         Arguments:
-            - channel: the name of the channel
+            - use_mock_redis_server: boolean indicating whether or not to use a mock Redis
+                connection that simulates talking to a Redis server by storing state internally
+
+        Redis Keyword Arguments:
+            - host: host of the Redis server
+            - port: port for the Redis server
+            - db: database number to use with the Redis server
+
+        For more Redis keyword arguments,
+        see: https://redis-py.readthedocs.io/en/stable/connections.html#redis.Redis
         """
-        self.logger.info(f"{self}:  Subscribing to channel '{channel}'")
-        self._pubsub.subscribe(**{channel: self._on_message})
+
+        # If indicated, use mock Redis server
+        if use_mock_redis_server:
+            self._connect_mock()
+
+        # Otherwise, try to connect to specified Redis server
+        else:
+            try:
+                self._connect_server(**redis_kwargs)
+
+            except redis.exceptions.RedisError as e:
+                host = redis_kwargs.get('host', 'localhost')
+                port = redis_kwargs.get('port', 6379)
+
+                # Try to spin up Redis server on localhost
+                if host is None or socket.gethostbyname(host) == '127.0.0.1':
+                    self.logger.warning(
+                        f"{self}:  Could not connect to Redis server at {host}:{port} - {e}")
+                    self.logger.warning(
+                        f"{self}:  Attempting to spin up Redis server at localhost:{port}")
+                    self._connect_localhost(**redis_kwargs)
+                else:
+                    raise e
+
+
+    def subscribe(self, *channels):
+        """
+        Subscribe to messages from one or more channels through the Redis connection.
+
+        Arguments:
+            - channels: tuple of channel names to subscribe to
+        """
+        self.logger.info(f"{self}:  Subscribing to channels: {channels}")
+        self._pubsub.subscribe(**{channel: self._on_message for channel in channels})
+
+
+    def unsubscribe(self, *channels):
+        """
+        Unsubscribe from the specified channels. If empty, unsubscribe from
+        all channels.
+
+        Arguments:
+            - channels: tuple of channel names to unsubscribe from
+        """
+        self.logger.info(f"{self}:  Unsubscribing from channels: {channels}")
+        self._pubsub.unsubscribe(*channels)
 
 
     def register(self, observer, channel):
@@ -130,7 +156,8 @@ class RedisBridge(Loggable):
             - observer: client object to receive messages
             - channel: the name of the channel on which the client should receive
         """
-        self.logger.debug(f"{self}:  Registering {observer} to receive messages on channel '{channel}'")
+        self.logger.debug(
+            f"{self}:  Registering {observer} to receive messages on channel '{channel}'")
 
         if not channel in self._observers.keys():
             self.subscribe(channel)
@@ -165,7 +192,8 @@ class RedisBridge(Loggable):
         # Unsuscribe to any channels that no longer have any observers
         for c in self._observers.keys():
             if len(self._observers[c]) == 0:
-                self.logger.info(f"{self}:  Unsubscribing from channel '{c}' -- no registered listeners")
+                self.logger.info(
+                    f"{self}:  Unsubscribing from channel '{c}' -- no registered listeners")
                 self._pubsub.unsubscribe(c)
 
 
@@ -203,9 +231,9 @@ class RedisBridge(Loggable):
 
         # Start the bridge
         self.logger.info(f"{self}:  Starting callback loop with RedisBridge")
-        self._connection.flushdb()
         if self._thread and self._thread.is_alive():
-            self.logger.warning(f"{self}:  Attempting to start RedisBridge that is already running")
+            self.logger.warning(
+                f"{self}:  Attempting to start RedisBridge that is already running")
         else:
             self._thread = self._pubsub.run_in_thread(sleep_time=sleep_time)
 
@@ -227,9 +255,6 @@ class RedisBridge(Loggable):
         if self._pubsub:
             self._pubsub.close()
 
-        if self._connection:
-            self._connection.flushdb()
-
 
     def send(self, data, channel):
         """
@@ -245,6 +270,7 @@ class RedisBridge(Loggable):
         # Create and send the message
         msg = Message(channel, data)
         self._connection.publish(channel, msg._encode())
+        return msg.id
 
 
     def request(self, data, channel, blocking=True, timeout=None):
@@ -271,12 +297,18 @@ class RedisBridge(Loggable):
 
         # If blocking, wait for a response
         try:
+            # Subscribe to channel
+            if channel.encode() not in self._pubsub.channels.items():
+                self.subscribe(channel)
+
+            # Get response
             response = self._responses[msg.id].get(timeout=timeout)
             return response
 
         # Raise a TimeoutError if no response received
         except queue.Empty:
-            e = TimeoutError(f"Request {data} on channel '{channel}' timed out after {timeout} seconds")
+            e = TimeoutError(
+                f"Request {data} on channel '{channel}' timed out after {timeout} seconds")
             self.logger.error(f"{self}:  {e}")
             raise e
 
@@ -307,7 +339,7 @@ class RedisBridge(Loggable):
             self._server_process.terminate()
 
 
-    def _connect(self, **kwargs):
+    def _connect_server(self, **kwargs):
         """
         Create and configure connection to Redis server.
         """
@@ -315,27 +347,42 @@ class RedisBridge(Loggable):
         port = kwargs.get('port', 6379)
         db = kwargs.get('db', 0)
 
+        self._connection = redis.Redis(**kwargs)
+        self._connection.ping()
+        self._pubsub = self._connection.pubsub(ignore_subscribe_messages=True)
+        self.logger.info(f"{self}:  Connected to Redis at {host}:{port}, database {db}")
+
+        # Set client pubsub hard / soft output buffer limits
+        # 1 GB hard limit, 64 MB per 60 seconds soft limit
+        self._connection.config_set('client-output-buffer-limit', f'pubsub {2**30} {2**26} 60')
+
+
+    def _connect_localhost(self, **kwargs):
+        """
+        Create an configure connection to local Redis server.
+        """
+        port = kwargs.get('port', 6379)
+
+        # Spin up a local Redis server
         try:
-            self._connection = redis.Redis(**kwargs)
-            self._connection.ping()
+            self._server_process = subprocess.Popen(['redis-server', '--port', str(port)])
 
-            # Set client pubsub hard / soft output buffer limits
-            # 1 GB hard limit, 64 MB per 60 seconds soft limit
-            self._connection.config_set('client-output-buffer-limit', f'pubsub {2**30} {2**26} 60')
+            # Wait for local Redis server
+            timeout = 1.0
+            start = time.time()
+            while check_server('localhost', port):
+                if time.time() - start > timeout:
+                    self._server_process.terminate()
+                    break
 
-            self.logger.info(f"{self}:  Connected to Redis at {host}:{port}, database {db}")
-            self._pubsub = self._connection.pubsub(ignore_subscribe_messages=True)
-
-        except redis.exceptions.ConnectionError as e:
-            self._connection = None
-            self._pubsub = None
-            self.logger.warning(f"{self}:  Could not connect to Redis server at {host}:{port} - {e}")
+        except FileNotFoundError as e:
+            self.logger.error(f"{self}:  Could not find executable 'redis-server'.")
 
         except Exception as e:
-            self._connection = None
-            self._pubsub = None
-            self.logger.warning(f"{self}:  Could not connect to Redis server at {host}:{port} - {e}")
-            raise e
+            self.logger.error(f"{self}:  Could not spin up Redis server - {e}")
+
+        # Connect to local Redis server
+        self._connect_server(**dict(kwargs, host='localhost'))
 
 
     def _connect_mock(self):
@@ -377,8 +424,9 @@ class RedisBridge(Loggable):
                 try:
                     observer._receive_redis(message)
                 except Exception as e:
-                    self.logger.exception(
-                        f"{self}:  Error in observer {observer} receiving message - {e}")
+                    self.logger.error(
+                        f"{self}:  Exception in observer {observer} receiving message.")
+                    self.logger.exception(f"{self}:  {e}")
 
 
     @property
