@@ -1,5 +1,7 @@
 import logging
+import random
 import socket
+import string
 
 
 
@@ -11,6 +13,14 @@ def check_server(host, port):
     return s.connect_ex((host, port))
 
 
+def uid(length=8):
+    """
+    Returns a unique string identifier.
+    """
+    alphabet = string.ascii_lowercase + string.digits
+    return ''.join(random.choices(alphabet, k=length))
+
+
 
 class Loggable:
     """
